@@ -1,29 +1,9 @@
-/*************************************************************************
- *
- * Copyright (c) 2011 Kohei Yoshida
- *
- * Permission is hereby granted, free of charge, to any person
- * obtaining a copy of this software and associated documentation
- * files (the "Software"), to deal in the Software without
- * restriction, including without limitation the rights to use,
- * copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following
- * conditions:
- *
- * The above copyright notice and this permission notice shall be
- * included in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
- * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
- * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
- * OTHER DEALINGS IN THE SOFTWARE.
- *
- ************************************************************************/
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/*
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ */
 
 #include "ixion/model_context.hpp"
 #include "ixion/formula_name_resolver.hpp"
@@ -151,8 +131,6 @@ bool set_shared_formula_tokens_to_cell(
 
 class model_context_impl
 {
-    typedef worksheet::column_type column_type;
-
     typedef boost::ptr_map<std::string, formula_cell> named_expressions_type;
     typedef boost::ptr_vector<std::string> strings_type;
     typedef boost::unordered_map<mem_str_buf, string_id_t, mem_str_buf::hash> string_map_type;
@@ -164,19 +142,6 @@ class model_context_impl
 public:
     model_context_impl(model_context& parent) :
         m_parent(parent),
-        m_max_row_size(1048576),
-        m_max_col_size(1024),
-        mp_config(new config),
-        mp_name_resolver(new formula_name_resolver_a1),
-        mp_cell_listener_tracker(new cell_listener_tracker(parent)),
-        mp_session_handler(new session_handler(parent))
-    {
-    }
-
-    model_context_impl(model_context& parent, row_t max_row_size, col_t max_col_size) :
-        m_parent(parent),
-        m_max_row_size(max_row_size),
-        m_max_col_size(max_col_size),
         mp_config(new config),
         mp_name_resolver(new formula_name_resolver_a1),
         mp_cell_listener_tracker(new cell_listener_tracker(parent)),
@@ -244,11 +209,13 @@ public:
     const string* get_named_expression_name(const formula_cell* expr) const;
     sheet_t get_sheet_index(const char* p, size_t n) const;
     std::string get_sheet_name(sheet_t sheet) const;
-    void append_sheet(const char* p, size_t n);
+    void append_sheet(const char* p, size_t n, row_t row_size, col_t col_size);
 
+    string_id_t append_string(const char* p, size_t n);
     string_id_t add_string(const char* p, size_t n);
     const std::string* get_string(string_id_t identifier) const;
     size_t get_string_count() const;
+    const column_store_t* get_column(sheet_t sheet, col_t col) const;
 
     const formula_tokens_t* get_formula_tokens(sheet_t sheet, size_t identifier) const;
     size_t add_formula_tokens(sheet_t sheet, formula_tokens_t* p);
@@ -276,9 +243,6 @@ public:
 private:
     model_context& m_parent;
 
-    row_t m_max_row_size;
-    col_t m_max_col_size;
-
     workbook m_sheets;
 
     config* mp_config;
@@ -348,10 +312,21 @@ std::string model_context_impl::get_sheet_name(sheet_t sheet) const
     return m_sheet_names[sheet];
 }
 
-void model_context_impl::append_sheet(const char* p, size_t n)
+void model_context_impl::append_sheet(const char* p, size_t n, row_t row_size, col_t col_size)
 {
     m_sheet_names.push_back(new string(p, n));
-    m_sheets.push_back(m_max_row_size, m_max_col_size);
+    m_sheets.push_back(row_size, col_size);
+}
+
+string_id_t model_context_impl::append_string(const char* p, size_t n)
+{
+    string_id_t str_id = m_strings.size();
+    std::auto_ptr<string> ps(new string(p, n));
+    p = &(*ps)[0];
+    mem_str_buf key(p, n);
+    m_strings.push_back(ps);
+    m_string_map.insert(string_map_type::value_type(key, str_id));
+    return str_id;
 }
 
 string_id_t model_context_impl::add_string(const char* p, size_t n)
@@ -360,13 +335,7 @@ string_id_t model_context_impl::add_string(const char* p, size_t n)
     if (itr != m_string_map.end())
         return itr->second;
 
-    string_id_t str_id = m_strings.size();
-    std::auto_ptr<string> ps(new string(p, n));
-    p = &(*ps)[0];
-    mem_str_buf key(p, n);
-    m_strings.push_back(ps);
-    m_string_map.insert(string_map_type::value_type(key, str_id));
-    return str_id;
+    return append_string(p, n);
 }
 
 const std::string* model_context_impl::get_string(string_id_t identifier) const
@@ -385,6 +354,19 @@ size_t model_context_impl::get_string_count() const
     return m_strings.size();
 }
 
+const column_store_t* model_context_impl::get_column(sheet_t sheet, col_t col) const
+{
+    if (static_cast<size_t>(sheet) >= m_sheets.size())
+        return NULL;
+
+    const worksheet& sh = m_sheets[sheet];
+
+    if (static_cast<size_t>(col) >= sh.size())
+        return NULL;
+
+    return &sh[col];
+}
+
 const formula_tokens_t* model_context_impl::get_formula_tokens(sheet_t sheet, size_t identifier) const
 {
     if (m_tokens.size() <= identifier)
@@ -532,8 +514,8 @@ void model_context_impl::set_shared_formula_range(sheet_t sheet, size_t identifi
 void model_context_impl::erase_cell(const abs_address_t& addr)
 {
     worksheet& sheet = m_sheets.at(addr.sheet);
-    worksheet::column_type& col_store = sheet.at(addr.column);
-    worksheet::column_type::iterator& pos_hint = sheet.get_pos_hint(addr.column);
+    column_store_t& col_store = sheet.at(addr.column);
+    column_store_t::iterator& pos_hint = sheet.get_pos_hint(addr.column);
 
     mdds::mtv::element_t celltype = col_store.get_type(addr.row);
     if (celltype == element_type_formula)
@@ -550,16 +532,16 @@ void model_context_impl::erase_cell(const abs_address_t& addr)
 void model_context_impl::set_numeric_cell(const abs_address_t& addr, double val)
 {
     worksheet& sheet = m_sheets.at(addr.sheet);
-    worksheet::column_type& col_store = sheet.at(addr.column);
-    worksheet::column_type::iterator& pos_hint = sheet.get_pos_hint(addr.column);
+    column_store_t& col_store = sheet.at(addr.column);
+    column_store_t::iterator& pos_hint = sheet.get_pos_hint(addr.column);
     pos_hint = col_store.set(pos_hint, addr.row, val);
 }
 
 void model_context_impl::set_boolean_cell(const abs_address_t& addr, bool val)
 {
     worksheet& sheet = m_sheets.at(addr.sheet);
-    worksheet::column_type& col_store = sheet.at(addr.column);
-    worksheet::column_type::iterator& pos_hint = sheet.get_pos_hint(addr.column);
+    column_store_t& col_store = sheet.at(addr.column);
+    column_store_t::iterator& pos_hint = sheet.get_pos_hint(addr.column);
     pos_hint = col_store.set(pos_hint, addr.row, val);
 }
 
@@ -567,16 +549,16 @@ void model_context_impl::set_string_cell(const abs_address_t& addr, const char*
 {
     worksheet& sheet = m_sheets.at(addr.sheet);
     string_id_t str_id = add_string(p, n);
-    worksheet::column_type& col_store = sheet.at(addr.column);
-    worksheet::column_type::iterator& pos_hint = sheet.get_pos_hint(addr.column);
+    column_store_t& col_store = sheet.at(addr.column);
+    column_store_t::iterator& pos_hint = sheet.get_pos_hint(addr.column);
     pos_hint = col_store.set(pos_hint, addr.row, str_id);
 }
 
 void model_context_impl::set_string_cell(const abs_address_t& addr, string_id_t identifier)
 {
     worksheet& sheet = m_sheets.at(addr.sheet);
-    worksheet::column_type& col_store = sheet.at(addr.column);
-    worksheet::column_type::iterator& pos_hint = sheet.get_pos_hint(addr.column);
+    column_store_t& col_store = sheet.at(addr.column);
+    column_store_t::iterator& pos_hint = sheet.get_pos_hint(addr.column);
     pos_hint = col_store.set(pos_hint, addr.row, identifier);
 }
 
@@ -592,8 +574,8 @@ void model_context_impl::set_formula_cell(const abs_address_t& addr, const char*
     }
 
     worksheet& sheet = m_sheets.at(addr.sheet);
-    worksheet::column_type& col_store = sheet.at(addr.column);
-    worksheet::column_type::iterator& pos_hint = sheet.get_pos_hint(addr.column);
+    column_store_t& col_store = sheet.at(addr.column);
+    column_store_t::iterator& pos_hint = sheet.get_pos_hint(addr.column);
     pos_hint = col_store.set(pos_hint, addr.row, fcell.release());
 }
 
@@ -604,24 +586,25 @@ void model_context_impl::set_formula_cell(
     fcell->set_shared(shared);
 
     worksheet& sheet = m_sheets.at(addr.sheet);
-    worksheet::column_type& col_store = sheet.at(addr.column);
-    worksheet::column_type::iterator& pos_hint = sheet.get_pos_hint(addr.column);
+    column_store_t& col_store = sheet.at(addr.column);
+    column_store_t::iterator& pos_hint = sheet.get_pos_hint(addr.column);
     pos_hint = col_store.set(pos_hint, addr.row, fcell.release());
 }
 
 abs_range_t model_context_impl::get_data_range(sheet_t sheet) const
 {
-    if (m_max_col_size <= 0 || m_max_row_size <= 0)
-        return abs_range_t(abs_range_t::invalid);
-
     const worksheet& cols = m_sheets.at(sheet);
     size_t col_size = cols.size();
     if (!col_size)
         return abs_range_t(abs_range_t::invalid);
 
+    row_t row_size = cols[0].size();
+    if (!row_size)
+        return abs_range_t(abs_range_t::invalid);
+
     abs_range_t range;
     range.first.column = 0;
-    range.first.row = m_max_row_size-1;
+    range.first.row = row_size-1;
     range.first.sheet = sheet;
     range.last.column = -1; // if this stays -1 all columns are empty.
     range.last.row = 0;
@@ -629,7 +612,7 @@ abs_range_t model_context_impl::get_data_range(sheet_t sheet) const
 
     for (size_t i = 0; i < col_size; ++i)
     {
-        const column_type& col = cols[i];
+        const column_store_t& col = cols[i];
         if (col.empty())
         {
             if (range.last.column < 0)
@@ -641,7 +624,7 @@ abs_range_t model_context_impl::get_data_range(sheet_t sheet) const
         {
             // First non-empty row.
 
-            column_type::const_iterator it = col.begin(), it_end = col.end();
+            column_store_t::const_iterator it = col.begin(), it_end = col.end();
             assert(it != it_end);
             if (it->type == mdds::mtv::element_type_empty)
             {
@@ -667,11 +650,11 @@ abs_range_t model_context_impl::get_data_range(sheet_t sheet) const
             range.last.column = i;
         }
 
-        if (range.last.row < (m_max_row_size-1))
+        if (range.last.row < (row_size-1))
         {
             // Last non-empty row.
 
-            column_type::const_reverse_iterator it = col.rbegin(), it_end = col.rend();
+            column_store_t::const_reverse_iterator it = col.rbegin(), it_end = col.rend();
             assert(it != it_end);
             if (it->type == mdds::mtv::element_type_empty)
             {
@@ -693,7 +676,7 @@ abs_range_t model_context_impl::get_data_range(sheet_t sheet) const
             }
             else
                 // Last block is not empty.
-                range.last.row = m_max_row_size - 1;
+                range.last.row = row_size - 1;
 
             range.last.column = i;
         }
@@ -735,7 +718,7 @@ celltype_t model_context_impl::get_celltype(const abs_address_t& addr) const
 
 double model_context_impl::get_numeric_value(const abs_address_t& addr) const
 {
-    const worksheet::column_type& col_store = m_sheets.at(addr.sheet).at(addr.column);
+    const column_store_t& col_store = m_sheets.at(addr.sheet).at(addr.column);
     switch (col_store.get_type(addr.row))
     {
         case mdds::mtv::element_type_numeric:
@@ -754,7 +737,7 @@ double model_context_impl::get_numeric_value(const abs_address_t& addr) const
 
 string_id_t model_context_impl::get_string_identifier(const abs_address_t& addr) const
 {
-    const worksheet::column_type& col_store = m_sheets.at(addr.sheet).at(addr.column);
+    const column_store_t& col_store = m_sheets.at(addr.sheet).at(addr.column);
     if (col_store.get_type(addr.row) != mdds::mtv::element_type_ulong)
         return empty_string_id;
 
@@ -763,7 +746,7 @@ string_id_t model_context_impl::get_string_identifier(const abs_address_t& addr)
 
 const formula_cell* model_context_impl::get_formula_cell(const abs_address_t& addr) const
 {
-    const worksheet::column_type& col_store = m_sheets.at(addr.sheet).at(addr.column);
+    const column_store_t& col_store = m_sheets.at(addr.sheet).at(addr.column);
     if (col_store.get_type(addr.row) != element_type_formula)
         return NULL;
 
@@ -772,7 +755,7 @@ const formula_cell* model_context_impl::get_formula_cell(const abs_address_t& ad
 
 formula_cell* model_context_impl::get_formula_cell(const abs_address_t& addr)
 {
-    worksheet::column_type& col_store = m_sheets.at(addr.sheet).at(addr.column);
+    column_store_t& col_store = m_sheets.at(addr.sheet).at(addr.column);
     if (col_store.get_type(addr.row) != element_type_formula)
         return NULL;
 
@@ -791,9 +774,6 @@ bool model_context::shared_tokens::operator== (const shared_tokens& r) const
 model_context::model_context() :
     mp_impl(new model_context_impl(*this)) {}
 
-model_context::model_context(row_t max_row_size, col_t max_col_size) :
-    mp_impl(new model_context_impl(*this, max_row_size, max_col_size)) {}
-
 model_context::~model_context()
 {
     delete mp_impl;
@@ -963,6 +943,11 @@ void model_context::set_shared_formula_range(sheet_t sheet, size_t identifier, c
     return mp_impl->set_shared_formula_range(sheet, identifier, range);
 }
 
+string_id_t model_context::append_string(const char* p, size_t n)
+{
+    return mp_impl->append_string(p, n);
+}
+
 string_id_t model_context::add_string(const char* p, size_t n)
 {
     return mp_impl->add_string(p, n);
@@ -1003,9 +988,9 @@ const string* model_context::get_named_expression_name(const formula_cell* expr)
     return mp_impl->get_named_expression_name(expr);
 }
 
-void model_context::append_sheet(const char* p, size_t n)
+void model_context::append_sheet(const char* p, size_t n, row_t row_size, col_t col_size)
 {
-    mp_impl->append_sheet(p, n);
+    mp_impl->append_sheet(p, n, row_size, col_size);
 }
 
 void model_context::set_session_handler(iface::session_handler* handler)
@@ -1018,4 +1003,10 @@ size_t model_context::get_string_count() const
     return mp_impl->get_string_count();
 }
 
+const column_store_t* model_context::get_column(sheet_t sheet, col_t col) const
+{
+    return mp_impl->get_column(sheet, col);
 }
+
+}
+/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
